@@ -58,8 +58,8 @@ def add_args(parser):
                         help="further limit selected hosts to an additional "
                              "pattern")
     parser.add_argument("--skip-tags", metavar="TAGS",
-                        help="only run plays and tasks whose tags do not match"
-                             "these values")
+                        help="only run plays and tasks whose tags do not "
+                             "match these values")
     parser.add_argument("-t", "--tags", metavar="TAGS",
                         help="only run plays and tasks tagged with these "
                              "values")
@@ -133,9 +133,13 @@ def build_args(parsed_args, playbooks,
         cmd += ["-e", "@%s" % vars_file]
     if parsed_args.extra_vars:
         for extra_var in parsed_args.extra_vars:
+            # Don't quote or escape variables passed via the kayobe -e CLI
+            # argument, to match Ansible's behaviour.
             cmd += ["-e", extra_var]
     if extra_vars:
         for extra_var_name, extra_var_value in extra_vars.items():
+            # Quote and escape variables originating within the python CLI.
+            extra_var_value = utils.quote_and_escape(extra_var_value)
             cmd += ["-e", "%s=%s" % (extra_var_name, extra_var_value)]
     if parsed_args.become:
         cmd += ["--become"]
@@ -197,7 +201,8 @@ def config_dump(parsed_args, host=None, hosts=None, var_name=None,
             extra_vars["dump_facts"] = facts
         # Don't use check mode for configuration dumps as we won't get any
         # results back.
-        run_playbook(parsed_args, "ansible/dump-config.yml",
+        playbook_path = utils.get_data_files_path("ansible", "dump-config.yml")
+        run_playbook(parsed_args, playbook_path,
                      extra_vars=extra_vars, tags=tags, check_output=True,
                      verbose_level=verbose_level, check=False)
         hostvars = {}
@@ -228,7 +233,9 @@ def install_galaxy_roles(parsed_args, force=False):
     :param force: Whether to force reinstallation of roles.
     """
     LOG.info("Installing galaxy role dependencies from kayobe")
-    utils.galaxy_install("requirements.yml", "ansible/roles", force=force)
+    requirements = utils.get_data_files_path("requirements.yml")
+    roles_destination = utils.get_data_files_path('ansible', 'roles')
+    utils.galaxy_install(requirements, roles_destination, force=force)
 
     # Check for requirements in kayobe configuration.
     kc_reqs_path = os.path.join(parsed_args.config_path,
