@@ -19,12 +19,21 @@ from cliff.command import Command
 
 from kayobe import ansible
 from kayobe import kolla_ansible
+from kayobe import utils
 from kayobe import vault
 
 
 def _build_playbook_list(*playbooks):
     """Return a list of names of playbook files given their basenames."""
-    return ["ansible/%s.yml" % playbook for playbook in playbooks]
+    return [
+        _get_playbook_path(playbook)
+        for playbook in playbooks
+    ]
+
+
+def _get_playbook_path(playbook):
+    """Return the absolute path of a playbook"""
+    return utils.get_data_files_path("ansible", "%s.yml" % playbook)
 
 
 class VaultMixin(object):
@@ -60,43 +69,17 @@ class KayobeAnsibleMixin(object):
             verbosity_args["quiet"] = True
         return verbosity_args
 
-    def run_kayobe_playbooks(self, parsed_args, *args, **kwargs):
+    def run_kayobe_playbooks(self, *args, **kwargs):
         kwargs.update(self._get_verbosity_args())
-        return ansible.run_playbooks(parsed_args, *args, **kwargs)
+        return ansible.run_playbooks(*args, **kwargs)
 
-    def run_kayobe_playbook(self, parsed_args, *args, **kwargs):
+    def run_kayobe_playbook(self, *args, **kwargs):
         kwargs.update(self._get_verbosity_args())
-        return ansible.run_playbook(parsed_args, *args, **kwargs)
+        return ansible.run_playbook(*args, **kwargs)
 
-    def run_kayobe_config_dump(self, parsed_args, *args, **kwargs):
+    def run_kayobe_config_dump(self, *args, **kwargs):
         kwargs.update(self._get_verbosity_args())
-        return ansible.config_dump(parsed_args, *args, **kwargs)
-
-    def generate_kolla_ansible_config(self, parsed_args, install=False,
-                                      service_config=True,
-                                      bifrost_config=False):
-        """Generate configuration for kolla-ansible.
-
-        :param install: If True, also install kolla-ansible.
-        :param service_config: If True, generate service configuration.
-        :param bifrost_config: If True, generate bifrost configuration.
-        """
-        # We use ignore_limit here because all of these plays execute against
-        # localhost, and are typically necessary for kolla-ansible to function
-        # correctly. Previously a number of people were caught out by using
-        # --limit and having these plays skipped.
-        tags = None if install else "config"
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags=tags,
-                                  ignore_limit=True)
-        if service_config:
-            playbooks = _build_playbook_list("kolla-openstack")
-            self.run_kayobe_playbooks(parsed_args, playbooks,
-                                      ignore_limit=True)
-        if bifrost_config:
-            playbooks = _build_playbook_list("kolla-bifrost")
-            self.run_kayobe_playbooks(parsed_args, playbooks,
-                                      ignore_limit=True)
+        return ansible.config_dump(*args, **kwargs)
 
 
 class KollaAnsibleMixin(object):
@@ -135,48 +118,21 @@ class KollaAnsibleMixin(object):
         return kolla_ansible.run_seed(*args, **kwargs)
 
 
-class ControlHostBootstrap(KayobeAnsibleMixin, KollaAnsibleMixin, VaultMixin,
-                           Command):
+class ControlHostBootstrap(KayobeAnsibleMixin, VaultMixin, Command):
     """Bootstrap the Kayobe control environment.
 
     * Downloads and installs Ansible roles from Galaxy.
     * Generates an SSH key for the Ansible control host, if one does not exist.
     * Installs kolla-ansible on the Ansible control host.
-    * Generates admin-openrc.sh and public-openrc.sh files when passwords.yml
-      exists.
     """
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Bootstrapping Kayobe Ansible control host")
         ansible.install_galaxy_roles(parsed_args)
         playbooks = _build_playbook_list("bootstrap")
-        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
-
-        passwords_exist = ansible.passwords_yml_exists(parsed_args)
-        if passwords_exist:
-            # Install and generate configuration - necessary for post-deploy.
-            ka_tags = None
-        else:
-            ka_tags = "install"
+        self.run_kayobe_playbooks(parsed_args, playbooks)
         playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags=ka_tags,
-                                  ignore_limit=True)
-
-        if passwords_exist:
-            # If we are bootstrapping a control host for an existing
-            # environment, we should also generate the admin-openrc.sh and
-            # public-openrc.sh scripts that provide admin credentials.
-
-            # FIXME: Fudge to work around incorrect configuration path.
-            extra_vars = {"node_config_directory":
-                          parsed_args.kolla_config_path}
-            self.run_kolla_ansible_overcloud(parsed_args, "post-deploy",
-                                             extra_vars=extra_vars)
-            # Create an environment file for accessing the public API as the
-            # admin user.
-            playbooks = _build_playbook_list("public-openrc")
-            self.run_kayobe_playbooks(parsed_args, playbooks,
-                                      ignore_limit=True)
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="install")
 
 
 class ControlHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
@@ -195,10 +151,9 @@ class ControlHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
         # Use force to upgrade roles.
         ansible.install_galaxy_roles(parsed_args, force=True)
         playbooks = _build_playbook_list("bootstrap")
-        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
+        self.run_kayobe_playbooks(parsed_args, playbooks)
         playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="install",
-                                  ignore_limit=True)
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="install")
 
 
 class ConfigurationDump(KayobeAnsibleMixin, VaultMixin, Command):
@@ -285,14 +240,19 @@ class PhysicalNetworkConfigure(KayobeAnsibleMixin, VaultMixin, Command):
         group.add_argument("--display", action="store_true",
                            help="display the candidate configuration and exit "
                                 "without applying it")
-        group.add_argument("--enable-discovery", action="store_true",
-                           help="configure the network for hardware discovery")
         group.add_argument("--interface-limit",
                            help="limit the switch interfaces to be configured "
                                 "by interface name")
         group.add_argument("--interface-description-limit",
                            help="limit the switch interfaces to be configured "
                                 "by interface description")
+        discovery = parser.add_mutually_exclusive_group()
+        discovery.add_argument("--enable-discovery", action="store_true",
+                               help="configure the network for hardware "
+                                    "discovery")
+        discovery.add_argument("--disable-discovery", action="store_true",
+                               help="deconfigure the network for hardware "
+                                    "discovery")
         return parser
 
     def take_action(self, parsed_args):
@@ -301,13 +261,16 @@ class PhysicalNetworkConfigure(KayobeAnsibleMixin, VaultMixin, Command):
         extra_vars["physical_network_display"] = parsed_args.display
         if parsed_args.enable_discovery:
             extra_vars["physical_network_enable_discovery"] = True
+        if parsed_args.disable_discovery:
+            extra_vars["physical_network_disable_discovery"] = True
         if parsed_args.interface_limit:
             extra_vars["physical_network_interface_limit"] = (
                 parsed_args.interface_limit)
         if parsed_args.interface_description_limit:
             extra_vars["physical_network_interface_description_limit"] = (
                 parsed_args.interface_description_limit)
-        self.run_kayobe_playbook(parsed_args, "ansible/physical-network.yml",
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path('physical-network'),
                                  limit=parsed_args.group,
                                  extra_vars=extra_vars)
 
@@ -320,15 +283,26 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
     * Add the host to SSH known hosts.
     * Configure a user account for use by kayobe for SSH access.
     * Optionally, create a virtualenv for remote target hosts.
+    * Optionally, wipe unmounted disk partitions (--wipe-disks).
     * Configure user accounts, group associations, and authorised SSH keys.
     * Configure a PyPI mirror.
     * Configure Yum repos.
     * Configure the host's network interfaces.
     * Set sysctl parameters.
     * Configure NTP.
+    * Configure LVM volumes.
     * Configure the host as a libvirt hypervisor.
     """
 
+    def get_parser(self, prog_name):
+        parser = super(SeedHypervisorHostConfigure, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Configuration")
+        group.add_argument("--wipe-disks", action='store_true',
+                           help="wipe partition and LVM data from all disks "
+                                "that are not mounted. Warning: this can "
+                                "result in the loss of data")
+        return parser
+
     def take_action(self, parsed_args):
         self.app.LOG.debug("Configuring seed hypervisor host OS")
         # Explicitly request the dump-config tag to ensure this play runs even
@@ -340,13 +314,44 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
             self.app.LOG.error("Could not determine kayobe_ansible_user "
                                "variable for seed hypervisor host")
             sys.exit(1)
-        playbooks = _build_playbook_list(
-            "ip-allocation", "ssh-known-host", "kayobe-ansible-user",
-            "pip", "kayobe-target-venv", "users", "yum", "dev-tools",
-            "network", "sysctl", "ntp", "seed-hypervisor-libvirt-host")
+
+        # Allocate IP addresses.
+        playbooks = _build_playbook_list("ip-allocation")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor")
 
+        playbooks = _build_playbook_list(
+            "ssh-known-host", "kayobe-ansible-user",
+            "pip", "kayobe-target-venv")
+        if parsed_args.wipe_disks:
+            playbooks += _build_playbook_list("wipe-disks")
+        playbooks += _build_playbook_list(
+            "users", "yum", "dev-tools", "network", "sysctl", "ntp", "lvm",
+            "seed-hypervisor-libvirt-host")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  limit="seed-hypervisor")
+
+
+class SeedHypervisorHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
+    """Run command on the seed hypervisor host."""
+
+    def get_parser(self, prog_name):
+        parser = super(SeedHypervisorHostCommandRun, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Host Command Run")
+        group.add_argument("--command", required=True,
+                           help="Command to run (required).")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Run command on seed hypervisor host")
+        extra_vars = {
+            "host_command_to_run": utils.escape_jinja(parsed_args.command)}
+        playbooks = _build_playbook_list("host-command-run")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  limit="seed-hypervisor",
+                                  extra_vars=extra_vars)
+
 
 class SeedHypervisorHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
     """Upgrade the seed hypervisor host services.
@@ -374,11 +379,15 @@ class SeedVMProvision(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Provisioning seed VM")
-        self.run_kayobe_playbook(parsed_args, "ansible/ip-allocation.yml",
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path("ip-allocation"),
                                  limit="seed")
-        self.run_kayobe_playbook(parsed_args, "ansible/seed-vm-provision.yml")
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path("seed-vm-provision"))
         # Now populate the Kolla Ansible inventory.
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path("kolla-ansible"),
+                                 tags="config")
 
 
 class SeedVMDeprovision(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
@@ -391,7 +400,7 @@ class SeedVMDeprovision(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     def take_action(self, parsed_args):
         self.app.LOG.debug("Deprovisioning seed VM")
         self.run_kayobe_playbook(parsed_args,
-                                 "ansible/seed-vm-deprovision.yml")
+                                 _get_playbook_path("seed-vm-deprovision"))
 
 
 class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
@@ -416,6 +425,7 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Optionally, create a virtualenv for kolla-ansible.
     * Configure a user account for kolla-ansible.
     * Configure Docker engine.
+    * Optionally, deploy a Docker Registry.
     """
 
     def get_parser(self, prog_name):
@@ -447,9 +457,13 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         python_interpreter = hostvars.get("ansible_python_interpreter")
         kolla_target_venv = hostvars.get("kolla_ansible_target_venv")
 
+        # Allocate IP addresses.
+        playbooks = _build_playbook_list("ip-allocation")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
+
         # Run kayobe playbooks.
         playbooks = _build_playbook_list(
-            "ip-allocation", "ssh-known-host", "kayobe-ansible-user",
+            "ssh-known-host", "kayobe-ansible-user",
             "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
@@ -457,8 +471,8 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             "users", "yum", "dev-tools", "disable-selinux", "network",
             "sysctl", "ip-routing", "snat", "disable-glean", "ntp", "lvm")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
-
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
         # Run kolla-ansible bootstrap-servers.
         # This command should be run as the kayobe ansible user because at this
@@ -488,6 +502,12 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars, limit="seed")
 
+        # Optionally, deploy a Docker Registry.
+        playbooks = _build_playbook_list("docker-registry")
+        extra_vars = {"kayobe_action": "deploy"}
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars, limit="seed")
+
 
 class SeedHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
     """Update packages on the seed host."""
@@ -514,6 +534,25 @@ class SeedHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
                                   extra_vars=extra_vars)
 
 
+class SeedHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
+    """Run command on the seed host."""
+
+    def get_parser(self, prog_name):
+        parser = super(SeedHostCommandRun, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Command Run")
+        group.add_argument("--command", required=True,
+                           help="Command to run (required).")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Run command on seed host")
+        extra_vars = {
+            "host_command_to_run": utils.escape_jinja(parsed_args.command)}
+        playbooks = _build_playbook_list("host-command-run")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed",
+                                  extra_vars=extra_vars)
+
+
 class SeedHostUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
                       Command):
     """Upgrade the seed host services.
@@ -547,9 +586,11 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Deploying seed services")
-        self.generate_kolla_ansible_config(parsed_args, service_config=False,
-                                           bifrost_config=True)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
+        playbooks = _build_playbook_list("kolla-bifrost")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
@@ -578,10 +619,11 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Upgrading seed services")
-        self.generate_kolla_ansible_config(parsed_args, service_config=False,
-                                           bifrost_config=True)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
         playbooks = _build_playbook_list(
+            "kolla-bifrost",
             "seed-service-upgrade-prep")
         self.run_kayobe_playbooks(parsed_args, playbooks)
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
@@ -668,12 +710,14 @@ class OvercloudInventoryDiscover(KayobeAnsibleMixin, VaultMixin, Command):
         # hosts will not be present in the following playbooks in which they
         # are used to populate other inventories.
         self.run_kayobe_playbook(parsed_args,
-                                 "ansible/overcloud-inventory-discover.yml")
+                                 _get_playbook_path(
+                                     "overcloud-inventory-discover"))
         # If necessary, allocate IP addresses for the discovered hosts.
         self.run_kayobe_playbook(parsed_args,
-                                 "ansible/ip-allocation.yml")
+                                 _get_playbook_path("ip-allocation"))
         # Now populate the Kolla Ansible inventory.
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
+        self.run_kayobe_playbook(parsed_args, _get_playbook_path(
+            "kolla-ansible"), tags="config")
 
 
 class OvercloudIntrospectionDataSave(KayobeAnsibleMixin, VaultMixin, Command):
@@ -814,9 +858,13 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         python_interpreter = hostvars.get("ansible_python_interpreter")
         kolla_target_venv = hostvars.get("kolla_ansible_target_venv")
 
+        # Allocate IP addresses.
+        playbooks = _build_playbook_list("ip-allocation")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
+
         # Kayobe playbooks.
         playbooks = _build_playbook_list(
-            "ip-allocation", "ssh-known-host", "kayobe-ansible-user",
+            "ssh-known-host", "kayobe-ansible-user",
             "pip", "kayobe-target-venv")
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
@@ -824,8 +872,8 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             "users", "yum", "dev-tools", "disable-selinux", "network",
             "sysctl", "disable-glean", "disable-cloud-init", "ntp", "lvm")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
-
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
         # Kolla-ansible bootstrap-servers.
         # The kolla-ansible bootstrap-servers command should be run as the
@@ -883,6 +931,25 @@ class OvercloudHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
                                   extra_vars=extra_vars)
 
 
+class OvercloudHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
+    """Run command on the overcloud host."""
+
+    def get_parser(self, prog_name):
+        parser = super(OvercloudHostCommandRun, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Command Run")
+        group.add_argument("--command", required=True,
+                           help="Command to run (required).")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Run command on overcloud host")
+        extra_vars = {
+            "host_command_to_run": utils.escape_jinja(parsed_args.command)}
+        playbooks = _build_playbook_list("host-command-run")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud",
+                                  extra_vars=extra_vars)
+
+
 class OvercloudHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
     """Upgrade the overcloud host services.
 
@@ -926,9 +993,10 @@ class OvercloudServiceConfigurationGenerate(KayobeAnsibleMixin,
         self.app.LOG.debug("Generating overcloud service configuration")
 
         # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
-        playbooks = _build_playbook_list("swift-setup")
+        playbooks = _build_playbook_list("kolla-openstack", "swift-setup")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before deployment.
@@ -957,6 +1025,12 @@ class OvercloudServiceConfigurationSave(KayobeAnsibleMixin, VaultMixin,
         parser = super(OvercloudServiceConfigurationSave, self).get_parser(
             prog_name)
         group = parser.add_argument_group("Service configuration")
+        group.add_argument("--exclude",
+                           help="optional comma-separated list of patterns "
+                                "matching filenames to exclude")
+        group.add_argument("--include",
+                           help="optional comma-separated list of patterns "
+                                "matching filenames to include")
         group.add_argument("--node-config-dir",
                            help="the directory to store the config files on "
                                 "the remote node (default /etc/kolla)")
@@ -969,6 +1043,10 @@ class OvercloudServiceConfigurationSave(KayobeAnsibleMixin, VaultMixin,
         self.app.LOG.debug("Saving overcloud service configuration")
         playbooks = _build_playbook_list("overcloud-service-config-save")
         extra_vars = {}
+        if parsed_args.exclude:
+            extra_vars["exclude_patterns"] = parsed_args.exclude
+        if parsed_args.include:
+            extra_vars["include_patterns"] = parsed_args.include
         if parsed_args.output_dir:
             extra_vars["config_save_path"] = parsed_args.output_dir
         if parsed_args.node_config_dir:
@@ -1004,9 +1082,10 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.app.LOG.debug("Deploying overcloud services")
 
         # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
-        playbooks = _build_playbook_list("swift-setup")
+        playbooks = _build_playbook_list("kolla-openstack", "swift-setup")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before deployment.
@@ -1020,7 +1099,7 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         playbooks = _build_playbook_list("overcloud-extras")
         extra_vars = {"kayobe_action": "deploy"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars)
+                                  extra_vars=extra_vars, limit="overcloud")
 
         # Post-deployment configuration.
         # FIXME: Fudge to work around incorrect configuration path.
@@ -1030,7 +1109,7 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Create an environment file for accessing the public API as the admin
         # user.
         playbooks = _build_playbook_list("public-openrc")
-        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
+        self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
 class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
@@ -1060,9 +1139,10 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.app.LOG.debug("Reconfiguring overcloud services")
 
         # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
-        playbooks = _build_playbook_list("swift-setup")
+        playbooks = _build_playbook_list("kolla-openstack", "swift-setup")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before reconfiguration.
@@ -1076,7 +1156,7 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         playbooks = _build_playbook_list("overcloud-extras")
         extra_vars = {"kayobe_action": "reconfigure"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars)
+                                  extra_vars=extra_vars, limit="overcloud")
 
         # Post-deployment configuration.
         # FIXME: Fudge to work around incorrect configuration path.
@@ -1086,7 +1166,7 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         # Create an environment file for accessing the public API as the admin
         # user.
         playbooks = _build_playbook_list("public-openrc")
-        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
+        self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
 class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
@@ -1115,7 +1195,8 @@ class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.app.LOG.debug("Upgrading overcloud services")
 
         # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args, install=True)
+        playbooks = _build_playbook_list("kolla-ansible", "kolla-openstack")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before upgrade.
         if not parsed_args.skip_prechecks:
@@ -1128,7 +1209,7 @@ class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
         playbooks = _build_playbook_list("overcloud-extras")
         extra_vars = {"kayobe_action": "upgrade"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars)
+                                  extra_vars=extra_vars, limit="overcloud")
 
 
 class OvercloudServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin,
@@ -1159,7 +1240,11 @@ class OvercloudServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.app.LOG.debug("Destroying overcloud services")
 
         # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+
+        playbooks = _build_playbook_list("kolla-openstack")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible destroy.
         extra_args = ["--yes-i-really-really-mean-it"]
@@ -1170,7 +1255,7 @@ class OvercloudServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin,
         playbooks = _build_playbook_list("overcloud-extras")
         extra_vars = {"kayobe_action": "destroy"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars)
+                                  extra_vars=extra_vars, limit="overcloud")
 
 
 class OvercloudContainerImagePull(KayobeAnsibleMixin, KollaAnsibleMixin,
@@ -1181,7 +1266,8 @@ class OvercloudContainerImagePull(KayobeAnsibleMixin, KollaAnsibleMixin,
         self.app.LOG.debug("Pulling overcloud container images")
 
         # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
 
         # Pull updated kolla container images.
         self.run_kolla_ansible_overcloud(parsed_args, "pull")
@@ -1190,7 +1276,7 @@ class OvercloudContainerImagePull(KayobeAnsibleMixin, KollaAnsibleMixin,
         playbooks = _build_playbook_list("overcloud-extras")
         extra_vars = {"kayobe_action": "pull"}
         self.run_kayobe_playbooks(parsed_args, playbooks,
-                                  extra_vars=extra_vars)
+                                  extra_vars=extra_vars, limit="overcloud")
 
 
 class OvercloudContainerImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
@@ -1254,6 +1340,7 @@ class OvercloudPostConfigure(KayobeAnsibleMixin, VaultMixin, Command):
       inspector service.
     * Register a provisioning network with glance.
     * Configure Grafana for control plane.
+    * Configure serial consoles for the ironic nodes
     """
 
     def take_action(self, parsed_args):
@@ -1261,7 +1348,8 @@ class OvercloudPostConfigure(KayobeAnsibleMixin, VaultMixin, Command):
         playbooks = _build_playbook_list(
             "overcloud-ipa-images", "overcloud-introspection-rules",
             "overcloud-introspection-rules-dell-lldp-workaround",
-            "provision-net", "overcloud-grafana-configure")
+            "provision-net", "overcloud-grafana-configure",
+            "baremetal-compute-serial-console-post-config")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
@@ -1316,6 +1404,57 @@ class BaremetalComputeRename(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
+class BaremetalComputeSerialConsoleBase(KayobeAnsibleMixin, VaultMixin,
+                                        Command):
+
+    """Base class for the baremetal serial console commands"""
+
+    @staticmethod
+    def process_limit(parsed_args, extra_vars):
+        if parsed_args.baremetal_compute_limit:
+            extra_vars["console_compute_node_limit"] = (
+                parsed_args.baremetal_compute_limit
+            )
+
+    def get_parser(self, prog_name):
+        parser = super(BaremetalComputeSerialConsoleBase, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Baremetal Serial Consoles")
+        group.add_argument("--baremetal-compute-limit",
+                           help="Limit the change to the hosts specified in "
+                                "this limit"
+                           )
+        return parser
+
+
+class BaremetalComputeSerialConsoleEnable(BaremetalComputeSerialConsoleBase):
+    """Enable Serial Console for Baremetal Compute Nodes"""
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Enabling serial console for ironic nodes")
+        extra_vars = {}
+        BaremetalComputeSerialConsoleBase.process_limit(parsed_args,
+                                                        extra_vars)
+        extra_vars["cmd"] = "enable"
+        playbooks = _build_playbook_list("baremetal-compute-serial-console")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
+
+
+class BaremetalComputeSerialConsoleDisable(BaremetalComputeSerialConsoleBase):
+    """Disable Serial Console for Baremetal Compute Nodes"""
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Disable serial console for ironic nodes")
+        extra_vars = {}
+        BaremetalComputeSerialConsoleBase.process_limit(parsed_args,
+                                                        extra_vars)
+        extra_vars["cmd"] = "disable"
+        playbooks = _build_playbook_list("baremetal-compute-serial-console")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  extra_vars=extra_vars)
+
+
 class BaremetalComputeUpdateDeploymentImage(KayobeAnsibleMixin, VaultMixin,
                                             Command):
     """Update the Ironic nodes to use the new  kernel and ramdisk images."""
